COUNTRIES = [
	'USA', 'CAN', 'GBR', 'DEU', 'FRA', 'POL', 'RUS', 'SWE',
	'TUR', 'BIH', 'ITA', 'LUX', 'CHE', 'BEL', 'GRC', 'NOR',
	'ESP', 'DNK', 'BGR', 'NLD', 'ROU', 'FIN', 'SRB', 'AUT',
	'HRV', 'PRT', 'SVK', 'CYP', 'MKD', 'SVN', 'UKR', 'HUN',
	'LTU', 'LVA', 'MLT', 'ISL', 'EST', 'CZE', 'JPN', 'KOR',
	'THA', 'HKG', 'CHN', 'TWN', 'MYS', 'SGP', 'ISR', 'IDN',
	'VNM', 'PHL', 'LKA', 'BGD', 'KWT', 'NPL', 'SAU', 'JOR',
	'OMN', 'IRN', 'IRQ', 'ARE', 'KAZ', 'BAH', 'AUS', 'NZL',
	'BRA', 'MEX', 'ARG', 'CHL', 'PER', 'COL', 'PAN', 'JAM',
	'VEN', 'ECU', 'ZAF', 'EGY', 'NGA', 'MUS', 'MAR', 'ZWE',
	'KEN', 'TUN', 'GHA', 'CIV', 'IND', 'PAK'
]